@@ -8,7 +8,8 @@
 #include "adw-clamp-scrollable.h"
 
 #include "adw-clamp-layout.h"
-#include "adw-macros-private.h"
+#include "adw-enums.h"
+#include "adw-length-unit.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -22,7 +23,7 @@
  * The primary use case for `AdwClampScrollable` is clamping
  * [class@Gtk.ListView].
  *
- * Since: 1.0
+ * See also: [class@ClampLayout].
  */
 
 enum {
@@ -30,6 +31,7 @@ enum {
   PROP_CHILD,
   PROP_MAXIMUM_SIZE,
   PROP_TIGHTENING_THRESHOLD,
+  PROP_UNIT,
 
   /* Overridden properties */
   PROP_ORIENTATION,
@@ -38,7 +40,7 @@ enum {
   PROP_HSCROLL_POLICY,
   PROP_VSCROLL_POLICY,
 
-  LAST_PROP = PROP_TIGHTENING_THRESHOLD + 1,
+  LAST_PROP = PROP_UNIT + 1,
 };
 
 struct _AdwClampScrollable
@@ -153,6 +155,9 @@ adw_clamp_scrollable_get_property (GObject    *object,
   case PROP_TIGHTENING_THRESHOLD:
     g_value_set_int (value, adw_clamp_scrollable_get_tightening_threshold (self));
     break;
+  case PROP_UNIT:
+    g_value_set_enum (value, adw_clamp_scrollable_get_unit (self));
+    break;
   case PROP_ORIENTATION:
     g_value_set_enum (value, self->orientation);
     break;
@@ -191,6 +196,9 @@ adw_clamp_scrollable_set_property (GObject      *object,
   case PROP_TIGHTENING_THRESHOLD:
     adw_clamp_scrollable_set_tightening_threshold (self, g_value_get_int (value));
     break;
+  case PROP_UNIT:
+    adw_clamp_scrollable_set_unit (self, g_value_get_enum (value));
+    break;
   case PROP_ORIENTATION:
     set_orientation (self, g_value_get_enum (value));
     break;
@@ -257,8 +265,6 @@ adw_clamp_scrollable_class_init (AdwClampScrollableClass *klass)
    * AdwClampScrollable:child: (attributes org.gtk.Property.get=adw_clamp_scrollable_get_child org.gtk.Property.set=adw_clamp_scrollable_set_child)
    *
    * The child widget of the `AdwClampScrollable`.
-   *
-   * Since: 1.0
    */
   props[PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -271,8 +277,6 @@ adw_clamp_scrollable_class_init (AdwClampScrollableClass *klass)
    * The maximum size allocated to the child.
    *
    * It is the width if the clamp is horizontal, or the height if it is vertical.
-   *
-   * Since: 1.0
    */
   props[PROP_MAXIMUM_SIZE] =
     g_param_spec_int ("maximum-size", NULL, NULL,
@@ -296,14 +300,27 @@ adw_clamp_scrollable_class_init (AdwClampScrollableClass *klass)
    *
    * Effectively, tightening the grip on the child before it reaches its maximum
    * size makes transitions to and from the maximum size smoother when resizing.
-   *
-   * Since: 1.0
    */
   props[PROP_TIGHTENING_THRESHOLD] =
     g_param_spec_int ("tightening-threshold", NULL, NULL,
                       0, G_MAXINT, 400,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwClampScrollable:unit: (attributes org.gtk.Property.get=adw_clamp_scrollable_get_unit org.gtk.Property.set=adw_clamp_scrollable_set_unit)
+   *
+   * The length unit for maximum size and tightening threshold.
+   *
+   * Allows the sizes to vary depending on the text scale factor.
+   *
+   * Since: 1.4
+   */
+  props[PROP_UNIT] =
+    g_param_spec_enum ("unit", NULL, NULL,
+                       ADW_TYPE_LENGTH_UNIT,
+                       ADW_LENGTH_UNIT_SP,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_layout_manager_type (widget_class, ADW_TYPE_CLAMP_LAYOUT);
@@ -341,8 +358,6 @@ adw_clamp_scrollable_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwClampScrollable`.
  *
  * Returns: the newly created `AdwClampScrollable`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_clamp_scrollable_new (void)
@@ -357,8 +372,6 @@ adw_clamp_scrollable_new (void)
  * Gets the child widget of @self.
  *
  * Returns: (nullable) (transfer none): the child widget of @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_clamp_scrollable_get_child (AdwClampScrollable *self)
@@ -374,8 +387,6 @@ adw_clamp_scrollable_get_child (AdwClampScrollable *self)
  * @child: (nullable): the child widget
  *
  * Sets the child widget of @self.
- *
- * Since: 1.0
  */
 void
 adw_clamp_scrollable_set_child (AdwClampScrollable *self,
@@ -384,6 +395,9 @@ adw_clamp_scrollable_set_child (AdwClampScrollable *self,
   g_return_if_fail (ADW_IS_CLAMP_SCROLLABLE (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (self->child == child)
     return;
 
@@ -429,8 +443,6 @@ adw_clamp_scrollable_set_child (AdwClampScrollable *self,
  * Gets the maximum size allocated to the child.
  *
  * Returns: the maximum size to allocate to the child
- *
- * Since: 1.0
  */
 int
 adw_clamp_scrollable_get_maximum_size (AdwClampScrollable *self)
@@ -452,8 +464,6 @@ adw_clamp_scrollable_get_maximum_size (AdwClampScrollable *self)
  * Sets the maximum size allocated to the child.
  *
  * It is the width if the clamp is horizontal, or the height if it is vertical.
- *
- * Since: 1.0
  */
 void
 adw_clamp_scrollable_set_maximum_size (AdwClampScrollable *self,
@@ -480,8 +490,6 @@ adw_clamp_scrollable_set_maximum_size (AdwClampScrollable *self,
  * Gets the size above which the child is clamped.
  *
  * Returns: the size above which the child is clamped
- *
- * Since: 1.0
  */
 int
 adw_clamp_scrollable_get_tightening_threshold (AdwClampScrollable *self)
@@ -514,8 +522,6 @@ adw_clamp_scrollable_get_tightening_threshold (AdwClampScrollable *self)
  *
  * Effectively, tightening the grip on the child before it reaches its maximum
  * size makes transitions to and from the maximum size smoother when resizing.
- *
- * Since: 1.0
  */
 void
 adw_clamp_scrollable_set_tightening_threshold (AdwClampScrollable *self,
@@ -534,3 +540,56 @@ adw_clamp_scrollable_set_tightening_threshold (AdwClampScrollable *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TIGHTENING_THRESHOLD]);
 }
+
+/**
+ * adw_clamp_scrollable_get_unit: (attributes org.gtk.Method.get_property=unit)
+ * @self: a clamp scrollable
+ *
+ * Gets the length unit for maximum size and tightening threshold.
+ *
+ * Returns: the length unit
+ *
+ * Since: 1.4
+ */
+AdwLengthUnit
+adw_clamp_scrollable_get_unit (AdwClampScrollable *self)
+{
+  AdwClampLayout *layout;
+
+  g_return_val_if_fail (ADW_IS_CLAMP_SCROLLABLE (self), ADW_LENGTH_UNIT_PX);
+
+  layout = ADW_CLAMP_LAYOUT (gtk_widget_get_layout_manager (GTK_WIDGET (self)));
+
+  return adw_clamp_layout_get_unit (layout);
+}
+
+/**
+ * adw_clamp_scrollable_set_unit: (attributes org.gtk.Method.set_property=unit)
+ * @self: a clamp
+ * @unit: the length unit
+ *
+ * Sets the length unit for maximum size and tightening threshold.
+ *
+ * Allows the sizes to vary depending on the text scale factor.
+ *
+ * Since: 1.4
+ */
+void
+adw_clamp_scrollable_set_unit (AdwClampScrollable *self,
+                               AdwLengthUnit       unit)
+{
+  AdwClampLayout *layout;
+
+  g_return_if_fail (ADW_IS_CLAMP_SCROLLABLE (self));
+  g_return_if_fail (unit >= ADW_LENGTH_UNIT_PX);
+  g_return_if_fail (unit <= ADW_LENGTH_UNIT_SP);
+
+  layout = ADW_CLAMP_LAYOUT (gtk_widget_get_layout_manager (GTK_WIDGET (self)));
+
+  if (adw_clamp_layout_get_unit (layout) == unit)
+    return;
+
+  adw_clamp_layout_set_unit (layout, unit);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_UNIT]);
+}
