@@ -8,6 +8,8 @@
 #include "adw-clamp.h"
 
 #include "adw-clamp-layout.h"
+#include "adw-enums.h"
+#include "adw-length-unit.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -32,13 +34,14 @@
  * If the child requires more than the requested maximum size, it will be
  * allocated the minimum size it can fit in instead.
  *
+ * `AdwClamp` can scale with the text scale factor, use the
+ * [property@Clamp:unit] property to enable that behavior.
+ *
+ * See also: [class@ClampLayout], [class@ClampScrollable].
+ *
  * ## CSS nodes
  *
  * `AdwClamp` has a single CSS node with name `clamp`.
- *
- * Its children will receive the style classes `.large` when the child reached
- * its maximum size, `.small` when the clamp allocates its full size to the
- * child, `.medium` in-between, or none if it hasn't computed its size yet.
  */
 
 enum {
@@ -46,11 +49,12 @@ enum {
   PROP_CHILD,
   PROP_MAXIMUM_SIZE,
   PROP_TIGHTENING_THRESHOLD,
+  PROP_UNIT,
 
   /* Overridden properties */
   PROP_ORIENTATION,
 
-  LAST_PROP = PROP_TIGHTENING_THRESHOLD + 1,
+  LAST_PROP = PROP_UNIT + 1,
 };
 
 struct _AdwClamp
@@ -106,6 +110,9 @@ adw_clamp_get_property (GObject    *object,
   case PROP_TIGHTENING_THRESHOLD:
     g_value_set_int (value, adw_clamp_get_tightening_threshold (self));
     break;
+  case PROP_UNIT:
+    g_value_set_enum (value, adw_clamp_get_unit (self));
+    break;
   case PROP_ORIENTATION:
     g_value_set_enum (value, self->orientation);
     break;
@@ -132,6 +139,9 @@ adw_clamp_set_property (GObject      *object,
   case PROP_TIGHTENING_THRESHOLD:
     adw_clamp_set_tightening_threshold (self, g_value_get_int (value));
     break;
+  case PROP_UNIT:
+    adw_clamp_set_unit (self, g_value_get_enum (value));
+    break;
   case PROP_ORIENTATION:
     set_orientation (self, g_value_get_enum (value));
     break;
@@ -211,6 +221,21 @@ adw_clamp_class_init (AdwClampClass *klass)
                       0, G_MAXINT, 400,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwClamp:unit: (attributes org.gtk.Property.get=adw_clamp_get_unit org.gtk.Property.set=adw_clamp_set_unit)
+   *
+   * The length unit for maximum size and tightening threshold.
+   *
+   * Allows the sizes to vary depending on the text scale factor.
+   *
+   * Since: 1.4
+   */
+  props[PROP_UNIT] =
+    g_param_spec_enum ("unit", NULL, NULL,
+                       ADW_TYPE_LENGTH_UNIT,
+                       ADW_LENGTH_UNIT_SP,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_layout_manager_type (widget_class, ADW_TYPE_CLAMP_LAYOUT);
@@ -286,6 +311,9 @@ adw_clamp_set_child (AdwClamp  *self,
   g_return_if_fail (ADW_IS_CLAMP (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (self->child == child)
     return;
 
@@ -403,3 +431,56 @@ adw_clamp_set_tightening_threshold (AdwClamp *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TIGHTENING_THRESHOLD]);
 }
+
+/**
+ * adw_clamp_get_unit: (attributes org.gtk.Method.get_property=unit)
+ * @self: a clamp
+ *
+ * Gets the length unit for maximum size and tightening threshold.
+ *
+ * Returns: the length unit
+ *
+ * Since: 1.4
+ */
+AdwLengthUnit
+adw_clamp_get_unit (AdwClamp *self)
+{
+  AdwClampLayout *layout;
+
+  g_return_val_if_fail (ADW_IS_CLAMP (self), ADW_LENGTH_UNIT_PX);
+
+  layout = ADW_CLAMP_LAYOUT (gtk_widget_get_layout_manager (GTK_WIDGET (self)));
+
+  return adw_clamp_layout_get_unit (layout);
+}
+
+/**
+ * adw_clamp_set_unit: (attributes org.gtk.Method.set_property=unit)
+ * @self: a clamp
+ * @unit: the length unit
+ *
+ * Sets the length unit for maximum size and tightening threshold.
+ *
+ * Allows the sizes to vary depending on the text scale factor.
+ *
+ * Since: 1.4
+ */
+void
+adw_clamp_set_unit (AdwClamp      *self,
+                    AdwLengthUnit  unit)
+{
+  AdwClampLayout *layout;
+
+  g_return_if_fail (ADW_IS_CLAMP (self));
+  g_return_if_fail (unit >= ADW_LENGTH_UNIT_PX);
+  g_return_if_fail (unit <= ADW_LENGTH_UNIT_SP);
+
+  layout = ADW_CLAMP_LAYOUT (gtk_widget_get_layout_manager (GTK_WIDGET (self)));
+
+  if (adw_clamp_layout_get_unit (layout) == unit)
+    return;
+
+  adw_clamp_layout_set_unit (layout, unit);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_UNIT]);
+}
