@@ -29,7 +29,7 @@
  *   `ADW_DIALOG_BOTTOM_SHEET` depending on available size.
  * @ADW_DIALOG_FLOATING: Present dialog as a centered floating window.
  * @ADW_DIALOG_BOTTOM_SHEET: Present dialog as a bottom sheet.
-
+ *
  * Describes the available presentation modes for [class@Dialog].
  *
  * New values may be added to this enumeration over time.
@@ -433,13 +433,13 @@ update_presentation (AdwDialog *self)
   if (priv->window)
     return;
 
-  g_object_ref (priv->child_breakpoint_bin);
-
   breakpoint =
     adw_breakpoint_bin_get_current_breakpoint (ADW_BREAKPOINT_BIN (priv->bin));
   use_bottom_sheet = priv->presentation_mode == ADW_DIALOG_BOTTOM_SHEET ||
                      breakpoint != NULL;
 
+  g_object_ref (priv->child_breakpoint_bin);
+
   root = gtk_widget_get_root (GTK_WIDGET (self));
 
   if (root) {
@@ -604,6 +604,14 @@ window_close_request_cb (AdwDialog *self)
   return GDK_EVENT_STOP;
 }
 
+static gboolean
+maybe_close_cb (GtkWidget *widget,
+                GVariant  *args,
+                AdwDialog *self)
+{
+  return adw_dialog_close (self);
+}
+
 static void
 present_as_window (AdwDialog *self,
                    GtkWidget *parent)
@@ -611,6 +619,8 @@ present_as_window (AdwDialog *self,
   AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
   GtkWidget *titlebar;
   GtkAccessibleRole role;
+  GtkEventController *shortcut_controller;
+  GtkShortcut *shortcut;
 
   if (priv->window) {
     gtk_window_present (GTK_WINDOW (priv->window));
@@ -624,6 +634,14 @@ present_as_window (AdwDialog *self,
   gtk_window_set_resizable (GTK_WINDOW (priv->window), FALSE);
   gtk_widget_add_css_class (priv->window, "dialog-window");
 
+  /* Esc to close */
+  shortcut = gtk_shortcut_new (gtk_keyval_trigger_new (GDK_KEY_Escape, 0),
+                               gtk_callback_action_new ((GtkShortcutFunc) maybe_close_cb, self, NULL));
+
+  shortcut_controller = gtk_shortcut_controller_new ();
+  gtk_shortcut_controller_add_shortcut (GTK_SHORTCUT_CONTROLLER (shortcut_controller), shortcut);
+  gtk_widget_add_controller (priv->window, shortcut_controller);
+
   if (parent) {
     GtkRoot *root = gtk_widget_get_root (parent);
 
